@@ -32,10 +32,7 @@ class Cache(object):
       if not shared.Settings.WASM_BACKEND:
         dirname = os.path.join(dirname, 'asmjs')
       else:
-        if shared.Settings.EXPERIMENTAL_USE_LLD:
-          dirname = os.path.join(dirname, 'wasm-lld')
-        else:
-          dirname = os.path.join(dirname, 'wasm')
+        dirname = os.path.join(dirname, 'wasm')
     self.dirname = dirname
     self.debug = debug
     self.acquired_count = 0
